package sample.publisher;

import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;

import com.poscoict.glueframework.websocket.GlueAbstractWebSocketHandler;
import com.poscoict.glueframework.websocket.GlueWebSocketDirectSender;

/**
 * WebSocket에 전송할 message를 생성하는 용도의 Thread
 * 설정된 handler에 등록된 모든 client들에게 message들이 전송된다.
 * 
 * */
public class AutoPublisher implements GlueWebSocketDirectSender, InitializingBean, DisposableBean, Runnable {
	GlueAbstractWebSocketHandler handler;
	String id;
	private Logger logger = LoggerFactory
			.getLogger(AutoPublisher.class);
	
	private Map<String, String> propertiesMap;
	private Thread thisThread;

	public AutoPublisher() {
		super();
		
	}
	
	@Override
	public void setWebSocketHandler(GlueAbstractWebSocketHandler handler) {
		this.handler = handler;
	}	
	
	@Override
	public void receiveDirectMessage(String msg) {
		//Do nothing
		logger.info("[RCV TXT from handler]" + msg);
	}
	

	@Override
	public void receiveDirectMessage(byte[] msg) {
		//Do nothing
	}
	
	@Override
	public void run() {

		String txtMsg = null;
		int index = 0;
		while (true) {

			txtMsg = "From Thread , msg : 你好  안녕하세요 test version :" + index;
			index++;
			
			try {
				logger.info(this.id + ":" +txtMsg);	
				if( handler != null ){
					this.handler.handleDirectMessage("TEST", txtMsg.getBytes("UTF-8") );	
				}
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			try {
				Thread.sleep(5000);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
		}
	}

	@Override
	public void afterPropertiesSet() throws Exception {
	
		String ip = propertiesMap.get("ip").toString();
		int port = -1;
		int timeout = 60000;
		
		try {
			port = Integer.parseInt((String)propertiesMap.get("port"));
		} catch (Exception e) {
			port = 12001;
		}
		
		try {
			timeout = Integer.parseInt((String)propertiesMap.get("timeout"));
		} catch (Exception e) {
			port = 12001;
		}
		
		this.thisThread = new Thread( this );
		this.thisThread.start();
		
		logger.info("*******************  [" + this.id  + "] AutoPublisher STARTED *******************");
	}
	
    public void setPropertiesMap( Map<String, String> propertiesMap )
    {
    	this.propertiesMap = propertiesMap;
    	this.id = propertiesMap.get("id");
    	if(id == null){
    		this.id = "Undefined";
    	}    	
    	
    	logger.info(" ******************* [" + this.id  + "]AutoPublisher setPropertiesMap *******************" + propertiesMap.toString());
    }

	@Override
	public void destroy() throws Exception {
		logger.info("AutoPublisher DESTROYED");
	}

	@Override
	public void receiveCommand(String cmd) {
		// TODO Auto-generated method stub
		if( "stop".equals(cmd)){
			this.thisThread.interrupt();
		}
		
	}

	@Override
	public GlueAbstractWebSocketHandler getHandler() {
		return this.handler;
	}
}
